#define MICO_CONF_IMR
#include <CORBA-SMALL.h>
#include <iostream.h>
#include <unistd.h>
#include "printer.h"
#include "load.h"

// create objects on demand when requested by a bind()
#define OBJECT_CREATION_ON_DEMAND

/*
 * PrinterFilter implementation
 */

static CORBA::ORB_var Forb;
static CORBA::BOA_var Fboa;
static char * bindAddr;

class PrinterFilter_impl : virtual public PrinterFilter_skel {
private:
	Printer_var p[4];	
	int load[5];
	void getPrinterRefs();
	int getLeastLoaded();
public:
    PrinterFilter_impl (const CORBA::BOA::ReferenceData &refdata)
	: PrinterFilter_skel (refdata)
    {
	    getPrinterRefs();
	    for(int i=0; i<5; i++)
		    load[i] = 0;
    }

    PrinterFilter_impl (CORBA::Object_ptr obj)
        : PrinterFilter_skel (obj)
    {
		CORBA::BOA_var boa = _boa();
		CORBA::BOA::ReferenceData_var id = boa->get_id (this);
		CORBA::String_var str = CORBA::ORB::tag_to_string (id);
	    getPrinterRefs();
    	for(int i=0; i<5; i++)
			load[i] = 0;
    }

	void setIdUp (CORBA::Long& id)
	{
	}
	
    void printUp (char *& text)
    {
	    int i=getLeastLoaded();
	    load[i] += strlen(text);
		
	    if( i )
	    {
		  p[i-1]->print(text);
		  setBounce();
	    }
    } 
};

int PrinterFilter_impl::getLeastLoaded() 
{
	int min = load[0];
	int idx = 0;

	for(int i=0; i<5; i++)
	{
		if(load[i] < min) 
		{
			min = load[i];
			idx = i;
		}
	}
	return idx;
}

void PrinterFilter_impl::getPrinterRefs(void)
{
	char *tags[4] = {"prn1", "prn2", "prn3", "prn4"};
	for(int i=0; i<4; i++)
	{
		CORBA::ORB::ObjectTag_var tag = CORBA::ORB::string_to_tag (tags[i]);
    	CORBA::Object_var obj = Forb->bind ("IDL:Printer:1.0", tag.in(), bindAddr);
    	assert (!CORBA::is_nil (obj));
    	p[i] = Printer::_narrow( obj );
		p[i]->setId(i+1);
	}
}

			
/*
 * PrinterFilter object restorer
 */

class PrinterFilterLoader : public CORBA::BOAObjectRestorer {
public:
    CORBA::Boolean restore (CORBA::Object_ptr obj)
    {
        if (!strcmp (obj->_repoid(), "IDL:PrinterFilter:1.0")) {
            (void)new PrinterFilter_impl (obj);
            return TRUE;
        }
        cout << "Cannot restore " << obj->_repoid() << " objects" << endl;
        return FALSE;
    }
#ifdef OBJECT_CREATION_ON_DEMAND
    CORBA::Boolean bind (const char *repoid, const CORBA::ORB::ObjectTag &tag)
    {
        if (!strcmp (repoid, "IDL:PrinterFilter:1.0")) {
	    cout << "Creating filter in bind... " << endl;
	    (void)new PrinterFilter_impl (tag);
	    return TRUE;
	}
	return FALSE;
    }
#endif
};


int main( int argc, char *argv[] )
{
    cout << "Filter init." << endl;
    PrinterFilterLoader loader;
    Forb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
    Fboa = Forb->BOA_init (argc, argv, "mico-local-boa");
	bindAddr = argv[1];

    if (!Fboa->restoring()) {
#ifndef OBJECT_CREATION_ON_DEMAND
	cout << "Creating filter in main... " << endl;
	CORBA::ORB::ObjectTag_var tag = CORBA::ORB::string_to_tag ("foobar");
	/*
	 * create an detector object whose ReferenceData equals "foobar".
	 */
        PrinterFilter_ptr acc = new PrinterFilter_impl (tag);
#endif
    }
    Fboa->impl_is_ready (CORBA::ImplementationDef::_nil());
    Forb->run ();

    return 0;
}
