#define MICO_CONF_IMR
#include <CORBA-SMALL.h>
#include <iostream.h>
#include <unistd.h>
#include <sys/time.h>
#include "printer.h"

// create objects on demand when requested by a bind()
#define OBJECT_CREATION_ON_DEMAND

/*
 * Printer implementation
 */

class Printer_impl : virtual public Printer_skel {
private:
	long _count;	
public:
    Printer_impl (const CORBA::BOA::ReferenceData &refdata)
	: Printer_skel (refdata)
    {
    }

    Printer_impl (CORBA::Object_ptr obj)
        : Printer_skel (obj)
    {
		CORBA::BOA_var boa = _boa();
		CORBA::BOA::ReferenceData_var id = boa->get_id (this);
		CORBA::String_var str = CORBA::ORB::tag_to_string (id);
    }

    void print(const char *text)
    {
	    cout << "Printer " << _count << " printing: " << text << endl << flush;
    }
	
    void setId(long id)
    {
		_count = id;
    }
};

/*
 * Printer object restorer
 */

class PrinterLoader : public CORBA::BOAObjectRestorer {
public:
    CORBA::Boolean restore (CORBA::Object_ptr obj)
    {
        if (!strcmp (obj->_repoid(), "IDL:Printer:1.0")) {
            (void)new Printer_impl (obj);
            return TRUE;
        }
        cout << "Cannot restore " << obj->_repoid() << " objects" << endl;
        return FALSE;
    }
#ifdef OBJECT_CREATION_ON_DEMAND
    CORBA::Boolean bind (const char *repoid, const CORBA::ORB::ObjectTag &tag)
    {
        if (!strcmp (repoid, "IDL:Printer:1.0")) {
	    cout << "Creating server in bind... " << endl;
	    (void)new Printer_impl (tag);
	    return TRUE;
	}
	return FALSE;
    }
#endif
};

int main( int argc, char *argv[] )
{
    cout << "Server init." << endl;

    PrinterLoader loader;
    CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
    CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");

    if (!boa->restoring()) {
#ifndef OBJECT_CREATION_ON_DEMAND
	cout << "Creating server in main... " << endl;
	CORBA::ORB::ObjectTag_var tag = CORBA::ORB::string_to_tag (argv[1]);
	/*
	 * create an printer object whose ReferenceData equals "foobar".
	 */
        Printer_ptr acc = new Printer_impl (tag);
#endif
    }
    boa->impl_is_ready (CORBA::ImplementationDef::_nil());
    orb->run ();

    return 0;
}
