#define MICO_CONF_IMR
#include <CORBA-SMALL.h>
#include <iostream.h>
#include <fstream.h>
#include "logger.h"

// create objects on demand when requested by a bind()
#define OBJECT_CREATION_ON_DEMAND

/*
 * Logger implementation
 */

static string outFile= "message.log";

class Logger_impl : virtual public Logger_skel {
public:
    Logger_impl (const CORBA::BOA::ReferenceData &refdata)
	: Logger_skel (refdata)
    {
    }

    Logger_impl (CORBA::Object_ptr obj)
        : Logger_skel (obj)
    {
		CORBA::BOA_var boa = _boa();
		CORBA::BOA::ReferenceData_var id = boa->get_id (this);
		CORBA::String_var str = CORBA::ORB::tag_to_string (id);
    }

    void log( char *&text, char *&ip)
    {
		ofstream out(outFile.c_str(), ios::app);
		if(!out)
		{
			cout << "Unable to open log file.\n" << flush;
			return;
		}
		out << "Msg Len: " << strlen(text) << "\tMsg: " << text << "\tTo: " << ip << endl << flush;
		out.close();
    }
};

/*
 * Logger object restorer
 */

class LoggerLoader : public CORBA::BOAObjectRestorer {
public:
    CORBA::Boolean restore (CORBA::Object_ptr obj)
    {
        if (!strcmp (obj->_repoid(), "IDL:Logger:1.0")) {
            (void)new Logger_impl (obj);
            return TRUE;
        }
        cout << "Cannot restore " << obj->_repoid() << " objects" << endl;
        return FALSE;
    }
#ifdef OBJECT_CREATION_ON_DEMAND
    CORBA::Boolean bind (const char *repoid, const CORBA::ORB::ObjectTag &tag)
    {
        if (!strcmp (repoid, "IDL:Logger:1.0")) {
	    cout << "Creating logger in bind... " << endl;
	    (void)new Logger_impl (tag);
	    return TRUE;
	}
	return FALSE;
    }
#endif
};

int main( int argc, char *argv[] )
{
    cout << "Logger init." << endl;

    LoggerLoader loader;
    CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
    CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");
 	//for(int i=0; i<argc; i++)
	//	cout << argv[i] << endl << flush;
	
	//outFile = argv[1];
    if (!boa->restoring()) {
#ifndef OBJECT_CREATION_ON_DEMAND
	cout << "Creating logger in main... " << endl;
	CORBA::ORB::ObjectTag_var tag = CORBA::ORB::string_to_tag ("foobar");
	/*
	 * create an mserver object whose ReferenceData equals "foobar".
	 */
        Logger_ptr acc = new Logger_impl (tag.in());
#endif
    }
    boa->impl_is_ready (CORBA::ImplementationDef::_nil());
    orb->run ();

    return 0;
}
