#define MICO_CONF_IMR
#include <CORBA-SMALL.h>
#include <iostream.h>
#include "mserver.h"

// create objects on demand when requested by a bind()
#define OBJECT_CREATION_ON_DEMAND

/*
 * MessageServer implementation
 */

class MessageServer_impl : virtual public MessageServer_skel {
public:
    MessageServer_impl (const CORBA::BOA::ReferenceData &refdata)
	: MessageServer_skel (refdata)
    {
    }

    MessageServer_impl (CORBA::Object_ptr obj)
        : MessageServer_skel (obj)
    {
		CORBA::BOA_var boa = _boa();
		CORBA::BOA::ReferenceData_var id = boa->get_id (this);
		CORBA::String_var str = CORBA::ORB::tag_to_string (id);
    }

    void send(const char *text, const char *ip)
    {
	    cout << "MessageServer: Sending message --> " << text << "\n\t To: "<< ip << endl << flush;
    }
};

/*
 * MessageServer object restorer
 */

class MessageServerLoader : public CORBA::BOAObjectRestorer {
public:
    CORBA::Boolean restore (CORBA::Object_ptr obj)
    {
        if (!strcmp (obj->_repoid(), "IDL:MessageServer:1.0")) {
            (void)new MessageServer_impl (obj);
            return TRUE;
        }
        cout << "Cannot restore " << obj->_repoid() << " objects" << endl;
        return FALSE;
    }
#ifdef OBJECT_CREATION_ON_DEMAND
    CORBA::Boolean bind (const char *repoid, const CORBA::ORB::ObjectTag &tag)
    {
        if (!strcmp (repoid, "IDL:MessageServer:1.0")) {
	    cout << "Creating server in bind... " << endl;
	    (void)new MessageServer_impl (tag);
	    return TRUE;
	}
	return FALSE;
    }
#endif
};

int main( int argc, char *argv[] )
{
    cout << "Server init." << endl;

    MessageServerLoader loader;
    CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
    CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");

    if (!boa->restoring()) {
#ifndef OBJECT_CREATION_ON_DEMAND
	cout << "Creating server in main... " << endl;
	CORBA::ORB::ObjectTag_var tag = CORBA::ORB::string_to_tag (argv[1]);
	/*
	 * create an mserver object whose ReferenceData equals "foobar".
	 */
        MessageServer_ptr acc = new MessageServer_impl (tag);
#endif
    }
    boa->impl_is_ready (CORBA::ImplementationDef::_nil());
    orb->run ();

    return 0;
}
