#define MICO_CONF_IMR
#include <CORBA-SMALL.h>
#include <iostream.h>
#include <fstream.h>
#include "money.h"
#include "filter.h"
 
// create objects on demand when requested by a bind()
#define OBJECT_CREATION_ON_DEMAND

/*
 * AccountFilter implementation
 */

class AccountFilter_impl : virtual public AccountFilter_skel {
    CORBA::Long _current_balance;
    CORBA::Long _calls;
public:
    AccountFilter_impl (const CORBA::BOA::ReferenceData &refdata)
	: AccountFilter_skel (refdata)
    {
        _calls = 0;
        _current_balance = 0;
    }

    AccountFilter_impl (CORBA::Object_ptr obj)
        : AccountFilter_skel (obj)
    {
        cout << "restoring filter id " << obj->_ident() << endl;
        _calls = 0;
        ifstream inp (obj->_ident());
        assert (inp);
        inp >> _current_balance;

	CORBA::BOA_var boa = _boa();
	CORBA::BOA::ReferenceData_var id = boa->get_id (this);
	CORBA::String_var str = CORBA::ORB::tag_to_string (id);
	cout << "filter ReferenceData = " << str.in() << endl;
    }

    CORBA::Boolean _save_object ()
    {
        cout << "saving filter id " << _ident() << endl;
        ofstream out (_ident());
        assert (out);
        out << _current_balance;
	return TRUE;
    }

    void check_exit ()
    {
        if (++_calls == 8000) {
            CORBA::BOA_var boa = _boa();
            CORBA::ORB_var orb = _orb();
	    /*
	     * beware: while calling deactivate_impl() we can get
	     * recursive invocations to the AccountFilter methods ...
	     */
            boa->deactivate_impl (CORBA::ImplementationDef::_nil());
	    orb->shutdown (TRUE);
        }
    }
  
    void depositUp1( Money& amount )
    {
		cerr << "Inside depositUp1 with amount = " << amount.cash << ". No change in amt.\n" << flush;
		amount.cash *= 1;
        check_exit ();
    }

    void depositUp2( Money& amount )
    {
		cerr << "Inside depositUp2 with amount = " << amount.cash << ". 2 times amt.\n" << flush;
		amount.cash *= 2;
        check_exit ();
    }

    void depositUp3( Money& amount )
    {
		cerr << "Inside depositUp3 with amount = " << amount.cash << ". 3 times amt.\n" << flush;
		amount.cash *= 3;
        check_exit ();
    }

    void depositUp4( Money& amount )
    {
		cerr << "Inside depositUp4 with amount = " << amount.cash << ". 4 times amt.\n" << flush;
		amount.cash *= 4;
        check_exit ();
    }

    void withdrawUp1( Money& amount )
	{
		cerr << "Inside withdrawUp1 with amount = " << amount.cash << ". Adding 50.\n" << flush;
		amount.cash += 50;
		check_exit ();
	}
    
    void withdrawUp2( Money& amount )
	{
		cerr << "Inside withdrawUp2 with amount = " << amount.cash << ". Adding 100.\n" << flush;
		amount.cash += 100;
		check_exit ();
	}
    
    void withdrawUp3( Money& amount )
	{
		cerr << "Inside withdrawUp3 with amount = " << amount.cash << ".Adding 150.\n" << flush;
		amount.cash += 150;
		check_exit ();
	}
    
    void withdrawUp4( Money& amount )
	{
		cerr << "Inside withdrawUp4 with amount = " << amount.cash << ". Adding 200.\n" << flush;
		amount.cash += 200;
		check_exit ();
	}
    
	Money balanceUp1( )
	{
		Money m;

		m.cash = 200;
		cerr << "Inside balanceUp1. Bouncing & Returning " << m.cash << "\n" << flush;
		setBounce();
		check_exit ();
		return m;
	}
	
	Money balanceUp2( )
	{
		Money m;

		m.cash = 400;
		cerr << "Inside balanceUp2. Returning " << m.cash << "\n" << flush;
		check_exit ();
		return m;
	}
	
	Money balanceUp3( )
	{
		Money m;

		m.cash = 600;
		cerr << "Inside balanceUp3. Bouncing & Returning " << m.cash << "\n" << flush;
		setBounce();
		check_exit ();
		return m;
	}
	
	Money balanceUp4( )
	{
		Money m;

		m.cash = 800;
		cerr << "Inside balanceUp4. Returning " << m.cash << "\n" << flush;
		check_exit ();
		return m;
	}
	
	Money balanceDown1( const Money& retVal )
	{
		cerr << "Inside balanceDown1. Got " << retVal.cash << "\n" << flush;
		Money m;
		m.cash = retVal.cash * 1;
		cerr << "Inside balanceDown1. Keeping same balance. Returning " << m.cash << "\n" << flush;
		return m;
	}
	
	Money balanceDown2( const Money& retVal )
	{
		cerr << "Inside balanceDown2. Got " << retVal.cash << "\n" << flush;
		Money m;
		m.cash = retVal.cash * 2;
		cerr << "Inside balanceDown2. Increasing balance 2 times. Returning " << m.cash << "\n" << flush;
		return m;
	}
	
	Money balanceDown3( const Money& retVal )
	{
		cerr << "Inside balanceDown3. Got " << retVal.cash << "\n" << flush;
		Money m;
		m.cash = retVal.cash * 3;
		cerr << "Inside balanceDown3. Increasing balance 3 times. Returning " << m.cash << "\n" << flush;
		return m;
	}
	
	Money balanceDown4( const Money& retVal )
	{
		cerr << "Inside balanceDown4. Got " << retVal.cash << "\n" << flush;
		Money m;
		m.cash = retVal.cash * 4;
		cerr << "Inside balanceDown4. Increasing balance 4 times. Returning " << m.cash << "\n" << flush;
		return m;
	}
	
};

/*
 * AccountFilter object restorer
 */

class AccountFilterLoader : public CORBA::BOAObjectRestorer {
public:
    CORBA::Boolean restore (CORBA::Object_ptr obj)
    {
        if (!strcmp (obj->_repoid(), "IDL:AccountFilter:1.0")) {
	    	cout << "CREATING FILTER IN RESTORE ... " << endl << flush;
            (void)new AccountFilter_impl (obj);
            return TRUE;
        }
        cout << "cannot restore filter " << obj->_repoid() << " objects" << endl;
        return FALSE;
    }
#ifdef OBJECT_CREATION_ON_DEMAND
    CORBA::Boolean bind (const char *repoid, const CORBA::ORB::ObjectTag &tag)
    {
        if (!strcmp (repoid, "IDL:AccountFilter:1.0")) {
	    cout << "CREATING FILTER IN BIND ... " << endl << flush;
	    (void)new AccountFilter_impl (tag);
	    return TRUE;
	}
	return FALSE;
    }
#endif
};


int main( int argc, char *argv[] )
{
    AccountFilterLoader loader;
    CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
    CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");

    if (!boa->restoring()) {
#ifndef OBJECT_CREATION_ON_DEMAND
	    cout << "CREATING FILTER IN MAIN... " << endl << flush;
	CORBA::ORB::ObjectTag_var tag = CORBA::ORB::string_to_tag ("foobar");
	/*
	 * create an account object whose ReferenceData equals "foobar".
	 */
        AccountFilter_ptr acc = new AccountFilter_impl (tag);
#endif
    }
    boa->impl_is_ready (CORBA::ImplementationDef::_nil());
    orb->run ();

    return 0;
}
