#define MICO_CONF_IMR
#include <CORBA-SMALL.h>
#include <iostream.h>
#include <fstream.h>
#include "filter1.h"
#include "money.h"
 
// create objects on demand when requested by a bind()
#define OBJECT_CREATION_ON_DEMAND

/*
 * AccountFilter implementation
 */

class AccountFilter1_impl : virtual public AccountFilter1_skel {
    CORBA::Long _current_balance;
    CORBA::Long _calls;
public:
    AccountFilter1_impl (const CORBA::BOA::ReferenceData &refdata)
	: AccountFilter1_skel (refdata)
    {
        _calls = 0;
        _current_balance = 0;
    }

    AccountFilter1_impl (CORBA::Object_ptr obj)
        : AccountFilter1_skel (obj)
    {
        cout << "restoring filter id " << obj->_ident() << endl;
        _calls = 0;
        ifstream inp (obj->_ident());
        assert (inp);
        inp >> _current_balance;

	CORBA::BOA_var boa = _boa();
	CORBA::BOA::ReferenceData_var id = boa->get_id (this);
	CORBA::String_var str = CORBA::ORB::tag_to_string (id);
	cout << "filter ReferenceData = " << str.in() << endl;
    }

    CORBA::Boolean _save_object ()
    {
        cout << "saving filter id " << _ident() << endl;
        ofstream out (_ident());
        assert (out);
        out << _current_balance;
	return TRUE;
    }

    void check_exit ()
    {
        if (++_calls == 8000) {
            CORBA::BOA_var boa = _boa();
            CORBA::ORB_var orb = _orb();
	    /*
	     * beware: while calling deactivate_impl() we can get
	     * recursive invocations to the AccountFilter methods ...
	     */
            boa->deactivate_impl (CORBA::ImplementationDef::_nil());
	    orb->shutdown (TRUE);
        }
    }
  
    void depositUp1( Money& amount )
    {
		cerr << "Inside depositUp1 with amount = " << amount.cash << ". 4 times amt.\n" << flush;
		amount.cash *= 4;
        check_exit ();
    }

    void withdrawUp1( Money& amount )
	{
		cerr << "Inside withdrawUp1 with amount = " << amount.cash << ". Adding 100.\n" << flush;
		amount.cash += 100;
		check_exit ();
	}
    
	Money balanceUp1( )
	{
		Money m;

		m.cash = 400;
		cerr << "Inside balanceUp1. Bouncing & Returning " << m.cash << "\n" << flush;
		setBounce();
		check_exit ();
		return m;
	}
	
	Money balanceDown1( const Money& retVal )
	{
		cerr << "Inside balanceDown1. Got " << retVal.cash << "\n" << flush;
		Money m;
		m.cash = retVal.cash * 4;
		cerr << "Inside balanceDown1. 4 times balance. Returning " << m.cash << "\n" << flush;
		return m;
	}
	
};

/*
 * AccountFilter object restorer
 */

class AccountFilterLoader : public CORBA::BOAObjectRestorer {
public:
    CORBA::Boolean restore (CORBA::Object_ptr obj)
    {
        if (!strcmp (obj->_repoid(), "IDL:AccountFilter1:1.0")) {
	    	cout << "CREATING FILTER1 IN RESTORE ... " << endl << flush;
            (void)new AccountFilter1_impl (obj);
            return TRUE;
        }
        cout << "cannot restore filter " << obj->_repoid() << " objects" << endl;
        return FALSE;
    }
#ifdef OBJECT_CREATION_ON_DEMAND
    CORBA::Boolean bind (const char *repoid, const CORBA::ORB::ObjectTag &tag)
    {
        if (!strcmp (repoid, "IDL:AccountFilter1:1.0")) {
	    cout << "CREATING FILTER1 IN BIND ... " << endl << flush;
	    (void)new AccountFilter1_impl (tag);
	    return TRUE;
	}
	return FALSE;
    }
#endif
};


int main( int argc, char *argv[] )
{
    AccountFilterLoader loader;
    CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
    CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");

    if (!boa->restoring()) {
#ifndef OBJECT_CREATION_ON_DEMAND
	    cout << "CREATING FILTER1 IN MAIN... " << endl << flush;
	CORBA::ORB::ObjectTag_var tag = CORBA::ORB::string_to_tag ("foobar");
	/*
	 * create an account object whose ReferenceData equals "foobar".
	 */
        AccountFilter1_ptr acc = new AccountFilter1_impl (tag);
#endif
    }
    boa->impl_is_ready (CORBA::ImplementationDef::_nil());
    orb->run ();

    return 0;
}
