#define MICO_CONF_IMR
#include <CORBA-SMALL.h>
#include <iostream.h>
#include <fstream.h>
#include "server.h"
 
// create objects on demand when requested by a bind()
#define OBJECT_CREATION_ON_DEMAND

/*
 * Server implementation
 */

class Server_impl : virtual public Server_skel {
	CORBA::Long _calls;
public:
    Server_impl (const CORBA::BOA::ReferenceData &refdata)
	: Server_skel (refdata)
    {
		_calls = 0;
    }

    Server_impl (CORBA::Object_ptr obj)
        : Server_skel (obj)
    {
		_calls = 0;
		CORBA::BOA_var boa = _boa();
		CORBA::BOA::ReferenceData_var id = boa->get_id (this);
		CORBA::String_var str = CORBA::ORB::tag_to_string (id);
		cout << "ReferenceData = " << str.in() << endl;
    }

    CORBA::Boolean _save_object ()
    {
		return TRUE;
    }

    void check_exit ()
    {
        if (++_calls == 8000) {
            CORBA::BOA_var boa = _boa();
            CORBA::ORB_var orb = _orb();
	    /*
	     * beware: while calling deactivate_impl() we can get
	     * recursive invocations to the Server methods ...
	     */
            boa->deactivate_impl (CORBA::ImplementationDef::_nil());
	    	orb->shutdown (TRUE);
        }
    }
  
    CORBA::Short test( CORBA::Long arg )
    {
		cerr << "Inside Local Server 'test'. arg = " << arg << ".\n" << flush;
        check_exit ();
		return static_cast<CORBA::Short>(arg + 20);
    }
};

/*
 * Server object restorer
 */

class ServerLoader : public CORBA::BOAObjectRestorer {
public:
    CORBA::Boolean restore (CORBA::Object_ptr obj)
    {
        if (!strcmp (obj->_repoid(), "IDL:Server:1.0")) {
	    	cout << "creating LOCAL SERVER in restore... " << endl << flush;
            (void)new Server_impl (obj);
            return TRUE;
        }
        cout << "cannot restore " << obj->_repoid() << " objects" << endl;
        return FALSE;
    }
#ifdef OBJECT_CREATION_ON_DEMAND
    CORBA::Boolean bind (const char *repoid, const CORBA::ORB::ObjectTag &tag)
    {
        if (!strcmp (repoid, "IDL:Server:1.0")) {
	    cout << "creating LOCAL SERVER in bind... " << endl << flush;
	    (void)new Server_impl (tag);
	    return TRUE;
	}
	return FALSE;
    }
#endif
};


int main( int argc, char *argv[] )
{
    ServerLoader loader;
    CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
    CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");

    if (!boa->restoring()) {
#ifndef OBJECT_CREATION_ON_DEMAND
	cout << "creating LOCAL SERVER in main... " << endl << flush;
	CORBA::ORB::ObjectTag_var tag = CORBA::ORB::string_to_tag ("local");
	/*
	 * create an account object whose ReferenceData equals "local".
	 */
        Server_ptr acc = new Server_impl (tag);
#endif
    }
    boa->impl_is_ready (CORBA::ImplementationDef::_nil());
    orb->run ();

    return 0;
}
