#define MICO_CONF_IMR
#include <CORBA-SMALL.h>
#include <iostream.h>
#include <fstream.h>
#include "filter.h"
 
// create objects on demand when requested by a bind()
#define OBJECT_CREATION_ON_DEMAND

/*
 * ServerFilter implementation
 */

class ServerFilter_impl : virtual public ServerFilter_skel {
	CORBA::Long _calls;
public:
    ServerFilter_impl (const CORBA::BOA::ReferenceData &refdata)
	: ServerFilter_skel (refdata)
    {
		_calls = 0;
    }

    ServerFilter_impl (CORBA::Object_ptr obj)
        : ServerFilter_skel (obj)
    {
		_calls = 0;
		CORBA::BOA_var boa = _boa();
		CORBA::BOA::ReferenceData_var id = boa->get_id (this);
		CORBA::String_var str = CORBA::ORB::tag_to_string (id);
		cout << "filter ReferenceData = " << str.in() << endl;
    }

    CORBA::Boolean _save_object ()
    {
		return TRUE;
    }

    void check_exit ()
    {
        if (++_calls == 8000) {
            CORBA::BOA_var boa = _boa();
            CORBA::ORB_var orb = _orb();
	    /*
	     * beware: while calling deactivate_impl() we can get
	     * recursive invocations to the ServerFilter methods ...
	     */
            boa->deactivate_impl (CORBA::ImplementationDef::_nil());
	    	orb->shutdown (TRUE);
        }
    }
  
	CORBA::Short testUp( CORBA::Long& arg )
	{
		cerr << "Inside Remote Filter 'testUp' with arg = " << arg << ".Returning same.\n" << flush;
        if( arg )
        {
            cerr << "BOUNCING\n" << flush;
            setBounce();
        }
		return static_cast<CORBA::Short>(arg);
	}

	CORBA::Short testDown( CORBA::Short result )
	{
		cerr << "Inside Remote Filter 'testDown' with result = " << result << ".Returning same.\n" << flush;
		return result;
	}

};

/*
 * ServerFilter object restorer
 */

class ServerFilterLoader : public CORBA::BOAObjectRestorer {
public:
    CORBA::Boolean restore (CORBA::Object_ptr obj)
    {
        if (!strcmp (obj->_repoid(), "IDL:ServerFilter:1.0")) {
	    	cout << "creating REMOTE FILTER in restore ... " << endl << flush;
            (void)new ServerFilter_impl (obj);
            return TRUE;
        }
        cout << "cannot restore filter " << obj->_repoid() << " objects" << endl;
        return FALSE;
    }
#ifdef OBJECT_CREATION_ON_DEMAND
    CORBA::Boolean bind (const char *repoid, const CORBA::ORB::ObjectTag &tag)
    {
        if (!strcmp (repoid, "IDL:ServerFilter:1.0")) {
	    cout << "creating REMOTE FILTER in bind ... " << endl << flush;
	    (void)new ServerFilter_impl (tag);
	    return TRUE;
	}
	return FALSE;
    }
#endif
};


int main( int argc, char *argv[] )
{
    ServerFilterLoader loader;
    CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
    CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");

    if (!boa->restoring()) {
#ifndef OBJECT_CREATION_ON_DEMAND
	    cout << "creating REMOTE FILTER in main... " << endl << flush;
	CORBA::ORB::ObjectTag_var tag = CORBA::ORB::string_to_tag ("remote");
	/*
	 * create an account object whose ReferenceData equals "remote".
	 */
        ServerFilter_ptr acc = new ServerFilter_impl (tag);
#endif
    }
    boa->impl_is_ready (CORBA::ImplementationDef::_nil());
    orb->run ();

    return 0;
}
