#define MICO_CONF_IMR
#include <CORBA-SMALL.h>
#include <iostream.h>
#include <fstream.h>
#include <unistd.h>
#include "money.h"
#include "filter.h"
#include <mico/CosNaming.h>

/*
 * AccountFilter implementation
 */

class AccountFilter_impl : virtual public AccountFilter_skel {
    CORBA::Long _current_balance;
    CORBA::Long _calls;
public:
    AccountFilter_impl ()
    {
        _calls = 0;
        _current_balance = 0;
    }

    AccountFilter_impl (CORBA::Object_ptr obj)
        : AccountFilter_skel (obj)
    {
        cout << "restoring id " << obj->_ident() << endl;
        _calls = 0;
        ifstream inp (obj->_ident());
        assert (inp);
        inp >> _current_balance;
    }

    CORBA::Boolean _save_object ()
    {
        cout << "saving id " << _ident() << endl;
        ofstream out (_ident());
        assert (out);
        out << _current_balance;
	return TRUE;
    }

    void check_exit ()
    {
        if (++_calls > 8000) {
            CORBA::ORB_var orb = _orb();
            CORBA::BOA_var boa = _boa();
	    /*
	     * beware: while calling deactivate_impl() we can get
	     * recursive invocations to the AccountFilter methods ...
	     */
            boa->deactivate_impl (CORBA::ImplementationDef::_nil());
	    orb->shutdown (TRUE);
        }
    }

    void depositUp( Money& amount )
    {
        cerr << "Inside depositUp with amount = " << amount.cash << ".\n" << flush;
        amount.cash *= 2;
        cerr << "Inside depositUp. Two times amount = " << amount.cash << ".\n" << flush;
        check_exit ();
    }

    void withdrawUp( Money& amount )
    {
        cerr << "Inside withdrawUp with amount = " << amount.cash << ".\n" << flush;
        amount.cash += 50;
        cerr << "Inside withdrawUp. Added 50, amount = " << amount.cash << ".\n" << flush;
        check_exit ();
    }

    Money balanceUp( )
    {
        Money m;

        m.cash = 200;
        cerr << "Inside balanceUp. Returning " << m.cash << "\n" << flush;
		check_exit ();
        return m;
    }

    Money balanceDown( const Money& retVal )
    {
        cerr << "Inside balanceDown. Got " << retVal.cash << "\n" << flush;
        Money m;
        m.cash = retVal.cash * 2;
        cerr << "Inside balanceDown. Increasing balance 2 times. Returning " << m.cash << "\n" << flush;
        return m;
    }

};

/*
 * AccountFilter object restorer
 */

class AccountFilterLoader : public CORBA::BOAObjectRestorer {
public:
    CORBA::Boolean restore (CORBA::Object_ptr obj)
    {
        if (!strcmp (obj->_repoid(), "IDL:AccountFilter:1.0")) {
            (void)new AccountFilter_impl (obj);
            return TRUE;
        }
        cout << "cannot restore " << obj->_repoid() << " objects" << endl;
        return FALSE;
    }
};


int main( int argc, char *argv[] )
{
  AccountFilterLoader loader;
  CORBA::ORB_var orb = CORBA::ORB_init( argc, argv, "mico-local-orb" );
  CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");
  
  if (!boa->restoring()) {
    cerr << "creating ..." << endl;
    AccountFilter_ptr acc = new AccountFilter_impl ();

    CORBA::Object_var nsobj =
      orb->resolve_initial_references ("NameService");
    assert (! CORBA::is_nil (nsobj));
  
    CosNaming::NamingContext_var nc = 
      CosNaming::NamingContext::_narrow (nsobj);

    CosNaming::Name name;
    name.length (1);
    name[0].id = CORBA::string_dup ("MyAccountFilter");
    name[0].kind = CORBA::string_dup ("");

    cerr << "binding name ..." << endl;
    nc->bind (name, acc);
  }
  boa->impl_is_ready (CORBA::ImplementationDef::_nil());
  orb->run ();

  return 0;
}
