%{

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define LEN 201

char return_type[2][LEN], function_name[LEN], interfaceName[LEN];
char up[LEN], down[LEN], filter[LEN];
int isDone;

FILE *fp1;
extern char *yytext;
extern FILE *yyin;

%}

%union{
	char string[201];
};

%token <string> INCLUDE
%token <string> INTERFACE ID 
%token FILTERABLE

%%
idlfile : includetextlist
		| filterableinterfacelist
		| includetextlist filterableinterfacelist

includetextlist : includetext
		| includetext includetextlist
		{
		 fprintf(fp1, "\n");
		}

includetext : INCLUDE
		{
		 fprintf(fp1, "%s", $1);
		}

filterableinterfacelist : filterableinterface
		| filterableinterface filterableinterfacelist
		;

filterableinterface : INTERFACE ID  '{'
		{	
		 fprintf(fp1,"%s %s%s %s%s","interface",$2,filter,"{","\n");
		 strcpy(interfaceName,$2);
		} 
		functionlist '}' ';' 
		{
			fprintf(fp1,"%s","};\n\n");
		}	
		;			
			
functionlist : function
		|  function functionlist
		;
		
function : ID ID ID '(' 
		{
			fprintf(fp1,"%s %s %s%s%s",$1,$2,$3,up,"(");
			strcpy(return_type[0],$1);
			strcpy(return_type[1],$2);
			strcpy(function_name,$3);
		}	
		arglist ')' ';'
		{
			fprintf(fp1,"%s",");\n");
			if(strcmp(return_type[0],"void"))
			{
fprintf(fp1,"%s %s %s%s",return_type[0],return_type[1],function_name,down);
fprintf(fp1,"%s %s %s %s","( in", return_type[0],return_type[1],"result );\n");
			}
		}
	| ID ID ID '(' ')' ';'
		{
			fprintf(fp1,"%s %s %s%s%s",$1,$2,$3,up,"(");
			strcpy(return_type[0],$1);
			strcpy(return_type[1],$2);
			strcpy(function_name,$3);
			fprintf(fp1,"%s",");\n");
			if(strcmp(return_type[0],"void"))
			{
fprintf(fp1,"%s %s %s%s",return_type[0],return_type[1],function_name,down);
fprintf(fp1,"%s %s %s %s","( in", return_type[0],return_type[1],"result );\n");
			}
		}
	| ID ID '(' 
		{
			fprintf(fp1,"%s %s%s%s",$1,$2,up,"(");
			strcpy(return_type[0],$1);
			strcpy(function_name,$2);
		}	
		arglist ')' ';'
		{
			fprintf(fp1,"%s",");\n");
			if(strcmp(return_type[0],"void"))
			{
		fprintf(fp1,"%s %s%s",return_type[0],function_name,down);
		fprintf(fp1,"%s %s %s","( in", return_type[0],"result );\n");
			}
		}
	| ID ID '(' ')' ';'
		{
			fprintf(fp1,"%s %s%s%s",$1,$2,up,"(");
			strcpy(return_type[0],$1);
			strcpy(function_name,$2);
			fprintf(fp1,"%s",");\n");
			if(strcmp(return_type[0],"void"))
			{
		fprintf(fp1,"%s %s%s",return_type[0],function_name,down);
		fprintf(fp1,"%s %s %s","( in", return_type[0],"result );\n");
			}
		}	
	;

arglist : argument
	| argument arglist
	;

argument : ID ID ID ID ','
	    {
	    	fprintf(fp1,"%s %s %s %s%s","inout",$2,$3,$4,",");
	    }  		
	| ID ID ID ID				
	    {
	    	fprintf(fp1,"%s %s %s %s","inout",$2,$3,$4);
	    }

	| ID ID ID ','
	    {
	    	fprintf(fp1,"%s %s %s%s","inout",$2,$3,",");
	    }  		
	| ID ID ID				
	    {
	    	fprintf(fp1,"%s %s %s","inout",$2,$3);
	    }
	;    

%%

void usage(void)
{
	printf("Usage: fidlgen (filter client idl | -h) [-f outputFile] [-i interfaceSuffix] [-u upSuffix] [-d downSuffix]\n");
	printf("\t-f outputFile: User specified file name for generated filter idl.\n");
	printf("\t-i interfaceSuffix: Suffix to be added to filter client interface name.\n");
	printf("\t-u upSuffix: Suffix to be added to every 'up' method.\n");
	printf("\t-d downSuffix: Suffix to be added to every 'down' method.\n");
	printf("\t-h: Show this message.\n");
	printf("Note: Client idl OR '-h' must be first argument. Other arguments can be specified in any order.\n");
}

int main(int argc, char *argv[])
{
	char ofile[LEN];
	int i;

	int yyerror(char *v);
	void yywrap();

	if( (argc < 2) || (!strcmp(argv[1],"-h")) ) 
	{
		usage();
		exit(0);
	}

	isDone = 1;
	strcpy( filter, "Filter" );
	strcpy( up, "Up" );
	strcpy( down, "Down" );
	strncpy(ofile, argv[1], strlen(argv[1])-4);
	ofile[strlen(argv[1])-4] = '\0';	
	strcat(ofile,"_filter.idl");

	for( i=2; i < argc; i+=2 )
	{
		if( (argv[i][0] != '-') || ((i+1) == argc) )
		{
			usage();
			exit(1);
		}
		switch( argv[i][1] )
		{
			case 'i':
				strcpy( filter, argv[i+1] );
				break;
			case 'u':
				strcpy( up, argv[i+1] );
				break;
			case 'd':
				strcpy( down, argv[i+1] );
				break;
			case 'f':
				strcpy( ofile, argv[i+1] );
				break;
			default:
				usage();
				exit(1);
				break;
		}
	}
	
	printf ("Opening %s\n", argv[1]);;
	yyin = fopen (argv[1],"r");
	if( !yyin )
	{
		fprintf( stderr, "Can't open file: %s\n", argv[1] );
		exit(1);
	}
	
	printf ("Opening %s\n", ofile);;
	fp1 = fopen(ofile,"w");
	if( !ofile )
	{
		fprintf( stderr, "Can't open file: %s\n", ofile );
		exit(1);
	}

	yyparse();

	fclose(fp1);
	fclose (yyin);
	
	if( isDone )
		printf("DONE.\n");
	else
		remove( ofile );

	return 0;
}

int yyerror(char *v) 
{
    printf ("Error %s.\n", v);
	isDone = 0;
	return 0;
}

void yywrap() 
{
}

