/*
 *  Filter Object Framework for MICO
 *	Pranav Nabar & Amit Padalkar
 *	8 Jan 2002
 */

#include <iostream>
#include <fstream>
#include <string>
#include <iosfwd>
#include <algorithm>
#include <vector>

using namespace std;

#define LEN 1000

const int INVALID = -1;

vector<string> tplate(2);
streampos start_pos;
vector<string> keywords;

void init(string s) throw(int)
{
	if( !s.length() )
		throw INVALID;

	tplate[0] = "class"+s+":virtualpublicCORBA::Object";
	tplate[1] = "class"+s+":publicvirtualCORBA::Object";

	keywords.push_back( "class" );
	keywords.push_back( s );
	keywords.push_back( s + ":" );
	keywords.push_back( s + ":virtual" );
	keywords.push_back( s + ":public" );
	keywords.push_back( ":" );
	keywords.push_back( ":virtual" );
	keywords.push_back( ":public" );
	keywords.push_back( "virtual" );
	keywords.push_back( "public" );
	keywords.push_back( "CORBA" );
	keywords.push_back( "CORBA::" );
	keywords.push_back( "CORBA::Object" );
	keywords.push_back( "::" );
	keywords.push_back( "::Object" );
	keywords.push_back( "Object" );
}

void skipSpace(fstream &fheader) throw(int)
{
    char ch;

    fheader.seekg(start_pos);
    while( fheader.get(ch) )
    {
        if( (ch != ' ') && (ch != '\t') && (ch != '\n') )
            break;
        else
            start_pos = fheader.tellg();
    }

    if( fheader.eof() )
    {
        throw INVALID;
    }

}

bool comment(fstream &fheader) throw(int)
{
    char prev, curr = '/';
    bool ret_val;

    fheader.seekg(start_pos);
    if( (!fheader.get(prev)) || (!fheader.get(curr)) )
        throw INVALID;

    if( (prev == '/') && (curr == '*') )
    {
        curr = prev;
        do
        {
            prev = curr;
            if( !fheader.get(curr) )
                throw INVALID;
        }
        while( !( (prev == '*') && (curr == '/') ) );
        start_pos = fheader.tellg();
        ret_val = true;
    }
    else
        ret_val = false;

    skipSpace(fheader);
    return ret_val;
}

string getWord(fstream &fheader) throw(int)
{
    string val;

    fheader.seekg(start_pos);

    if( !(fheader >> val) )
		throw INVALID;

    start_pos = fheader.tellg();

    return val;
}

string getToken(fstream &fheader, streampos &pos) throw(int)
{
    string val;

    if( fheader.eof() )
        throw INVALID;

	do
	{
    	skipSpace(fheader);
    	while( comment(fheader) ) ;

		pos = start_pos;
		val = getWord(fheader);
	}
	while( find(keywords.begin(), keywords.end(), val) == keywords.end() );

    return val;
}

int main(int argc, char *argv[])
{
	if(argc != 3)
	{
		cout << "Usage: posttranslate <IDL generated filter header file> <filter class name>" << endl;
		exit(0);
	}
	
	fstream fheader(argv[1], ios::in|ios::out);
	start_pos = 0;
	
	if(!fheader) 
	{
		cout << "Can't open filter header file: " << argv[1] << endl;
		exit(0);
	}

	try
	{
		init(argv[2]);
	}
	catch( int )
	{
		cout << "Specify filter class name.\n" << flush;
	}
	
	cout << "Translating filter header file: " << argv[1] << " for class: " << argv[2] << endl;

	try
	{
		string searchStr, val;
		streampos keyPos;
	
		for( int i=0; i < 4; ++i )
		{
			val = getToken( fheader, keyPos );
			searchStr += val;
		}

		while( (searchStr.find(tplate[0]) == string::npos) && (searchStr.find(tplate[1]) == string::npos) )
		{
			val = getToken( fheader, keyPos );
			searchStr += val;
		}
	
		val.replace( val.find("Object"), 6, "Filter" );
		fheader.seekp( keyPos );
		fheader << val;

		cout << "Translation done." << endl;
	}
	catch( int )
	{
		cout << "Translation Error.\n" << flush;
	}

	fheader.close();

	return 0;
}

